@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: scardcontrol.c,v 1.5 2006-04-22 16:04:28 rousseau Exp $
+ * $Id: scardcontrol.c,v 1.1.1.7 2006-08-11 18:38:23 rousseau Exp $
  */
 
 #include <stdio.h>
@@ -26,6 +26,7 @@
 #include <sys/time.h>
 #include <unistd.h>
 #include <string.h>
+#include <arpa/inet.h>
 #include <winscard.h>
 #include <reader.h>
 
@@ -228,9 +229,9 @@ int main(int argc, char *argv[])
 	for (i = 0; i < length; i++)
 	{
 		if (pcsc_tlv[i].tag == FEATURE_VERIFY_PIN_DIRECT)
-			verify_ioctl = pcsc_tlv[i].value;
+			verify_ioctl = ntohl(pcsc_tlv[i].value);
 		if (pcsc_tlv[i].tag == FEATURE_MODIFY_PIN_DIRECT)
-			modify_ioctl = pcsc_tlv[i].value;
+			modify_ioctl = ntohl(pcsc_tlv[i].value);
 	}
 
 	if (0 == verify_ioctl)
@@ -340,8 +341,8 @@ int main(int argc, char *argv[])
 
 		FD_ZERO(&fd);
 		FD_SET(STDIN_FILENO, &fd);	/* stdin */
-		timeout.tv_sec = 0;			/* timeout = 0 */
-		timeout.tv_usec = 0;
+		timeout.tv_sec = 0;			/* timeout = 0.1s */
+		timeout.tv_usec = 100000;
 
 		/* we only try to read stdin if the pinpad is on a keyboard
 		 * we do not read stdin for a SPR 532 for example */
@@ -484,8 +485,8 @@ int main(int argc, char *argv[])
 		{
 			FD_ZERO(&fd);
 			FD_SET(STDIN_FILENO, &fd);	/* stdin */
-			timeout.tv_sec = 0;			/* timeout = 0 */
-			timeout.tv_usec = 0;
+			timeout.tv_sec = 0;			/* timeout = 0.1s */
+			timeout.tv_usec = 100000;
 
 			/* we only try to read stdin if the pinpad is on a keyboard
 			 * we do not read stdin for a SPR 532 for example */
