@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c 2014 2006-04-18 15:05:33Z rousseau $
+ * $Id: ccid_usb.c 2119 2006-08-01 20:05:22Z rousseau $
  */
 
 #define __CCID_USB__
@@ -107,11 +107,11 @@ struct _bogus_firmware
 };
 
 static struct _bogus_firmware Bogus_firmwares[] = {
-	{ 0x0b97, 0x7762, 0x0111 },	/* Oz776S */ /* the firmware version if not correct since I don't have received a working reader yet */
+	{ 0x0b97, 0x7762, 0x0111 },	/* Oz776S */ /* the firmware version is not correct since I do not have received a working reader yet */
 	{ 0x04e6, 0xe001, 0x0516 },	/* SCR 331 */
 	{ 0x04e6, 0x5111, 0x0620 },	/* SCR 331-DI */
 	{ 0x04e6, 0x5115, 0x0514 },	/* SCR 335 */
-	{ 0x04e6, 0xe003, 0x0504 },	/* SPR 532 */
+	{ 0x04e6, 0xe003, 0x0510 },	/* SPR 532 */
 	{ 0x0D46, 0x3001, 0x0037 },	/* KAAN Base */
 	{ 0x0D46, 0x3002, 0x0037 },	/* KAAN Advanced */
 	{ 0x09C3, 0x0008, 0x0203 },	/* ActivCard V2 */
@@ -249,7 +249,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 	else
 		return STATUS_UNSUCCESSFUL;
 	vendorID = strlen(keyValue);
-	alias = 0x1D;
+	alias = 0x1C;
 	for (; vendorID--;)
 		alias ^= keyValue[vendorID];
 
@@ -493,7 +493,9 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 {
 	int rv;
 	char debug_header[] = "<- 121234 ";
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
+read_again:
 	sprintf(debug_header, "<- %06X ", (int)reader_index);
 
 	rv = usb_bulk_read(usbDevice[reader_index].handle,
@@ -503,9 +505,15 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 	if (rv < 0)
 	{
 		*length = 0;
-		DEBUG_CRITICAL4("usb_bulk_read(%s/%s): %s",
-			usbDevice[reader_index].dev->bus->dirname,
-			usbDevice[reader_index].dev->filename, strerror(errno));
+		if (usbDevice[reader_index].dev->bus)
+		{
+			DEBUG_CRITICAL4("usb_bulk_read(%s/%s): %s",
+					usbDevice[reader_index].dev->bus->dirname,
+					usbDevice[reader_index].dev->filename, strerror(errno));
+		}
+		else
+			DEBUG_CRITICAL2("usb_bulk_read(no device): %s", strerror(errno));
+
 		return STATUS_UNSUCCESSFUL;
 	}
 
@@ -513,6 +521,14 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 
 	DEBUG_XXD(debug_header, buffer, *length);
 
+#define BSEQ_OFFSET 6
+	if ((*length >= BSEQ_OFFSET)
+		&& (buffer[BSEQ_OFFSET] < *ccid_descriptor->pbSeq -1))
+	{
+		DEBUG_INFO("Duplicate frame detected");
+		goto read_again;
+	}
+
 	return STATUS_SUCCESS;
 } /* ReadUSB */
 
