@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.c 1986 2006-03-28 06:55:31Z rousseau $
+ * $Id: commands.c 2120 2006-08-03 10:04:01Z rousseau $
  */
 
 #include <string.h>
@@ -41,8 +41,13 @@
 
 #define max( a, b )   ( ( ( a ) > ( b ) ) ? ( a ) : ( b ) )
 #define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
+#define IFD_ERROR_INSUFFICIENT_BUFFER 700
 
 /* internal functions */
+static RESPONSECODE CmdXfrBlockAPDU_extended(unsigned int reader_index,
+	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
+	unsigned char rx_buffer[]);
+
 static RESPONSECODE CmdXfrBlockTPDU_T0(unsigned int reader_index,
 	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
 	unsigned char rx_buffer[]);
@@ -284,7 +289,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	i2dw(a - 10, cmd + 1);  /* CCID message length */
 
 	old_read_timeout = ccid_descriptor -> readTimeout;
-	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]);	/* at least 30 seconds */
+	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]+10);	/* at least 30 seconds */
 
 	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
 	{
@@ -292,7 +297,7 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 		return IFD_COMMUNICATION_ERROR;
 	}
 
-	ret = CCID_Receive(reader_index, RxLength, RxBuffer);
+	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
 
 	/* T=1 Protocol Management for a TPDU reader */
 	if ((IFD_SUCCESS == ret)
@@ -504,7 +509,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	i2dw(a - 10, cmd + 1);	/* command length (includes bPINOperation) */
 
 	old_read_timeout = ccid_descriptor -> readTimeout;
-	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]);	/* at least 30 seconds */
+	ccid_descriptor -> readTimeout = max(30, TxBuffer[0]+10);	/* at least 30 seconds */
 
 	if (WritePort(reader_index, a, cmd) != STATUS_SUCCESS)
 	{
@@ -512,7 +517,7 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
  		return IFD_COMMUNICATION_ERROR;
 	}
 
- 	ret = CCID_Receive(reader_index, RxLength, RxBuffer);
+ 	ret = CCID_Receive(reader_index, RxLength, RxBuffer, NULL);
 
 	/* T=1 Protocol Management for a TPDU reader */
 	if ((IFD_SUCCESS == ret)
@@ -713,24 +718,6 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
-	/* command length too big for CCID reader? */
-	if (tx_length > ccid_descriptor->dwMaxCCIDMessageLength)
-	{
-		DEBUG_CRITICAL3("Command too long (%d bytes) for max: %d bytes",
-			tx_length, ccid_descriptor->dwMaxCCIDMessageLength);
-		return_value = IFD_COMMUNICATION_ERROR;
-		goto clean_up_and_return;
-	}
-
-	/* command length too big for CCID driver? */
-	if (tx_length > CMD_BUF_SIZE)
-	{
-		DEBUG_CRITICAL3("Command too long (%d bytes) for max: %d bytes",
-			tx_length, CMD_BUF_SIZE);
-		return_value = IFD_COMMUNICATION_ERROR;
-		goto clean_up_and_return;
-	}
-
 	/* APDU or TPDU? */
 	switch (ccid_descriptor->dwFeatures & CCID_CLASS_EXCHANGE_MASK)
 	{
@@ -747,13 +734,17 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 			break;
 
 		case CCID_CLASS_SHORT_APDU:
-		case CCID_CLASS_EXTENDED_APDU:
-			/* We only support extended APDU if the reader can support the
-			 * command length. See test above */
 			return_value = CmdXfrBlockTPDU_T0(reader_index,
 				tx_length, tx_buffer, rx_length, rx_buffer);
 			break;
 
+		case CCID_CLASS_EXTENDED_APDU:
+			return_value = CmdXfrBlockAPDU_extended(reader_index,
+				tx_length, tx_buffer, rx_length, rx_buffer);
+			if (return_value != IFD_SUCCESS)
+				*rx_length = 0;
+			break;
+
 		case CCID_CLASS_CHARACTER:
 			if (protocol == T_0)
 				return_value = CmdXfrBlockCHAR_T0(reader_index, tx_length,
@@ -771,7 +762,6 @@ RESPONSECODE CmdXfrBlock(unsigned int reader_index, unsigned int tx_length,
 			return_value = IFD_COMMUNICATION_ERROR;
 	}
 
-clean_up_and_return:
 	return return_value;
 } /* CmdXfrBlock */
 
@@ -797,7 +787,10 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 
 	/* check that the command is not too large */
 	if (tx_length > CMD_BUF_SIZE)
+	{
+		DEBUG_CRITICAL2("TX Length too big: %d", tx_length);
 		return IFD_NOT_SUPPORTED;
+	}
 
 	memcpy(cmd+10, tx_buffer, tx_length);
 
@@ -814,10 +807,11 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
  *
  ****************************************************************************/
 RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
-	unsigned char rx_buffer[])
+	unsigned char rx_buffer[], unsigned char *chain_parameter)
 {
 	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
 	unsigned int length;
+	RESPONSECODE return_value = IFD_SUCCESS;
 
 time_request:
 	length = sizeof(cmd);
@@ -827,6 +821,13 @@ time_request:
 		return IFD_COMMUNICATION_ERROR;
 	}
 
+	if (length < STATUS_OFFSET)
+	{
+		DEBUG_CRITICAL2("Not enough data received: %d bytes", length);
+		*rx_length = 0;
+		return IFD_COMMUNICATION_ERROR;
+	}
+
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
 		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
@@ -865,16 +866,158 @@ time_request:
 	}
 
 	length = dw2i(cmd, 1);
-	if (length < *rx_length)
+	if (length <= *rx_length)
 		*rx_length = length;
 	else
+	{
+		DEBUG_CRITICAL2("overrun by %d bytes", length - *rx_length);
 		length = *rx_length;
+		return_value = IFD_ERROR_INSUFFICIENT_BUFFER;
+	}
 	memcpy(rx_buffer, cmd+10, length);
 
-	return IFD_SUCCESS;
+	/* Extended case?
+	 * Only valid for RDR_to_PC_DataBlock frames */
+	if (chain_parameter)
+		*chain_parameter = cmd[CHAIN_PARAMETER_OFFSET];
+
+	return return_value;
 } /* CCID_Receive */
 
 
+/*****************************************************************************
+ *
+ *					CmdXfrBlockAPDU_extended
+ *
+ ****************************************************************************/
+static RESPONSECODE CmdXfrBlockAPDU_extended(unsigned int reader_index,
+	unsigned int tx_length, unsigned char tx_buffer[], unsigned int *rx_length,
+	unsigned char rx_buffer[])
+{
+	RESPONSECODE return_value;
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	unsigned char chain_parameter;
+	unsigned int local_tx_length, sent_length;
+	unsigned int local_rx_length, received_length;
+	int buffer_overflow = 0;
+
+	DEBUG_COMM2("T=0 (extended): %d bytes", tx_length);
+
+	/* send the APDU */
+	sent_length = 0;
+
+	/* we suppose one command is enough */
+	chain_parameter = 0x00;
+
+	local_tx_length = tx_length - sent_length;
+	if (local_tx_length > CMD_BUF_SIZE)
+	{
+		local_tx_length = CMD_BUF_SIZE;
+		/* the command APDU begins with this command, and continue in the next
+		 * PC_to_RDR_XfrBlock */
+		chain_parameter = 0x01;
+	}
+	if (local_tx_length > ccid_descriptor->dwMaxCCIDMessageLength-10)
+	{
+		local_tx_length = ccid_descriptor->dwMaxCCIDMessageLength-10;
+		chain_parameter = 0x01;
+	}
+
+send_next_block:
+	return_value = CCID_Transmit(reader_index, local_tx_length, tx_buffer,
+		chain_parameter, 0);
+	if (return_value != IFD_SUCCESS)
+		return return_value;
+
+	sent_length += local_tx_length;
+	tx_buffer += local_tx_length;
+
+	/* we just sent the last block (0x02) or only one block was needded (0x00) */
+	if ((0x02 == chain_parameter) || (0x00 == chain_parameter))
+		goto receive_block;
+
+	/* read a nul block */
+	return_value = CCID_Receive(reader_index, &local_rx_length, NULL, NULL);
+	if (return_value != IFD_SUCCESS)
+		return return_value;
+
+	/* size of the next block */
+	if (tx_length - sent_length > local_tx_length)
+	{
+		/* the abData field continues a command APDU and
+		 * another block is to follow */
+		chain_parameter = 0x03;
+	}
+	else
+	{
+		/* this abData field continues a command APDU and ends
+		 * the APDU command */
+		chain_parameter = 0x02;
+
+		/* last (smaller) block */
+		local_tx_length = tx_length - sent_length;
+	}
+
+	goto send_next_block;
+
+receive_block:
+	/* receive the APDU */
+	received_length = 0;
+
+receive_next_block:
+	local_rx_length = *rx_length - received_length;
+	return_value = CCID_Receive(reader_index, &local_rx_length, rx_buffer,
+		&chain_parameter);
+	if (IFD_ERROR_INSUFFICIENT_BUFFER == return_value)
+	{
+		buffer_overflow = 1;
+
+		/* we continue to read all the response APDU */
+		return_value = IFD_SUCCESS;
+	}
+
+	if (return_value != IFD_SUCCESS)
+		return return_value;
+
+	/* advance in the reiceiving buffer */
+	rx_buffer += local_rx_length;
+	received_length += local_rx_length;
+
+	switch (chain_parameter)
+	{
+		/* the response APDU begins and ends in this command */
+		case 0x00:
+		/* this abData field continues the response APDU and ends the response
+		 * APDU */
+		case 0x02:
+			break;
+
+		/* the response APDU begins with this command and is to continue */
+		case 0x01:
+		/* this abData field continues the response APDU and another block is
+		 * to follow */
+		case 0x03:
+		/* empty abData field, continuation of the command APDU is expected in
+		 * next PC_to_RDR_XfrBlock command */
+		case 0x10:
+			/* send a nul block */
+			return_value = CCID_Transmit(reader_index, 0, NULL, 0, 0);
+			if (return_value != IFD_SUCCESS)
+				return return_value;
+
+			goto receive_next_block;
+	}
+
+	*rx_length = received_length;
+
+	/* generate an overflow detected by pcscd */
+	if (buffer_overflow)
+		(*rx_length)++;
+
+	return IFD_SUCCESS;
+} /* CmdXfrBlockAPDU_extended */
+
+
 /*****************************************************************************
  *
  *					CmdXfrBlockTPDU_T0
@@ -885,14 +1028,31 @@ static RESPONSECODE CmdXfrBlockTPDU_T0(unsigned int reader_index,
 	unsigned char rx_buffer[])
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
 	DEBUG_COMM2("T=0: %d bytes", tx_length);
 
+	/* command length too big for CCID reader? */
+	if (tx_length > ccid_descriptor->dwMaxCCIDMessageLength-10)
+	{
+		DEBUG_CRITICAL3("Command too long (%d bytes) for max: %d bytes",
+				tx_length, ccid_descriptor->dwMaxCCIDMessageLength-10);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
+	/* command length too big for CCID driver? */
+	if (tx_length > CMD_BUF_SIZE)
+	{
+		DEBUG_CRITICAL3("Command too long (%d bytes) for max: %d bytes",
+				tx_length, CMD_BUF_SIZE);
+		return IFD_COMMUNICATION_ERROR;
+	}
+
 	return_value = CCID_Transmit(reader_index, tx_length, tx_buffer, 0, 0);
 	if (return_value != IFD_SUCCESS)
 		return return_value;
 
-	return CCID_Receive(reader_index, rx_length, rx_buffer);
+	return CCID_Receive(reader_index, rx_length, rx_buffer, NULL);
 } /* CmdXfrBlockTPDU_T0 */
 
 
@@ -995,7 +1155,7 @@ static RESPONSECODE T0ProcACK(unsigned int reader_index,
 		if (return_value != IFD_SUCCESS)
 			return return_value;
 
-		return_value = CCID_Receive(reader_index, &ret_len, tmp_buf);
+		return_value = CCID_Receive(reader_index, &ret_len, tmp_buf, NULL);
 		if (return_value != IFD_SUCCESS)
 			return return_value;
 
@@ -1057,7 +1217,7 @@ static RESPONSECODE T0ProcSW1(unsigned int reader_index,
 
 		in_len = 1;
 
-		return_value = CCID_Receive(reader_index, &in_len, tmp_buf);
+		return_value = CCID_Receive(reader_index, &in_len, tmp_buf, NULL);
 		if (return_value != IFD_SUCCESS)
 			return return_value;
 
@@ -1148,7 +1308,7 @@ static RESPONSECODE CmdXfrBlockCHAR_T0(unsigned int reader_index,
 		if (in_len == 0)
 		{
 			in_len = 1;
-			return_value = CCID_Receive(reader_index, &in_len, tmp_buf);
+			return_value = CCID_Receive(reader_index, &in_len, tmp_buf, NULL);
 			if (return_value != IFD_SUCCESS)
 			{
 				DEBUG_CRITICAL("CCID_Receive failed");
@@ -1228,7 +1388,7 @@ static RESPONSECODE CmdXfrBlockTPDU_T1(unsigned int reader_index,
 	RESPONSECODE return_value = IFD_SUCCESS;
 	int ret;
 
-	DEBUG_COMM2("T=1: %d bytes", tx_length);
+	DEBUG_COMM3("T=1: %d and %d bytes", tx_length, *rx_length);
 
 	ret = t1_transceive(&((get_ccid_slot(reader_index)) -> t1), 0,
 		tx_buffer, tx_length, rx_buffer, *rx_length);
@@ -1284,7 +1444,11 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 		if (0x00 == cmd[ERROR_OFFSET])	/* command not supported */
 			return IFD_NOT_SUPPORTED;
 		else
-			return IFD_COMMUNICATION_ERROR;
+			if ((cmd[ERROR_OFFSET] >= 1) && (cmd[ERROR_OFFSET] <= 127))
+				/* a parameter is not changeable */
+				return IFD_SUCCESS;
+			else
+				return IFD_COMMUNICATION_ERROR;
 	}
 
 	return IFD_SUCCESS;
