@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: defs.h 1808 2006-01-18 10:25:34Z rousseau $
+ * $Id: defs.h 2123 2006-08-07 10:05:40Z rousseau $
  */
 
 #include <pcsclite.h>
@@ -61,8 +61,8 @@ typedef enum {
 /* Flag set when a power down is requested */
 #define MASK_POWERFLAGS_PDWN 0x02
 
-/* Communication buffer size (max=cmd+adpu+Lc+data+Le) */
-#define CMD_BUF_SIZE (1+4+1+256+1)
+/* Communication buffer size (max=adpu+Lc+data+Le) */
+#define CMD_BUF_SIZE (4+1+256+1)
 /* Larger communication buffer size (max=reader status+data+sw) */
 #define RESP_BUF_SIZE (1+256+2)
 
