@@ -17,11 +17,12 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-/* $Id: ifdhandler.c 2019 2006-04-20 18:57:14Z rousseau $ */
+/* $Id: ifdhandler.c 2130 2006-08-09 07:17:31Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <arpa/inet.h>
 #include "misc.h"
 #include <pcsclite.h>
 #include <ifdhandler.h>
@@ -65,7 +66,7 @@ static unsigned int T1_card_timeout(double f, double d, int TC1, int BWI,
 	int CWI, int clock_frequency);
 
 
-EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPTSTR lpcDevice)
+EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPSTR lpcDevice)
 {
 	RESPONSECODE return_value = IFD_SUCCESS;
 	int reader_index;
@@ -98,9 +99,27 @@ EXTERNAL RESPONSECODE IFDHCreateChannelByName(DWORD Lun, LPTSTR lpcDevice)
 		ReleaseReaderIndex(reader_index);
 	}
 	else
+	{
 		/* Maybe we have a special treatment for this reader */
 		ccid_open_hack(reader_index);
 
+		/* Try to access the reader */
+		/* This "warm up" sequence is sometimes needed when pcscd is
+		 * restarted with the reader already connected. We get some
+		 * "usb_bulk_read: Resource temporarily unavailable" on the first
+		 * few tries. It is an empirical hack */
+		if ((IFD_COMMUNICATION_ERROR == IFDHICCPresence(Lun))
+			&& (IFD_COMMUNICATION_ERROR == IFDHICCPresence(Lun))
+			&& (IFD_COMMUNICATION_ERROR == IFDHICCPresence(Lun)))
+		{
+			DEBUG_CRITICAL("failed");
+			return_value = IFD_COMMUNICATION_ERROR;
+
+			/* release the allocated reader_index */
+			ReleaseReaderIndex(reader_index);
+		}
+	}
+
 #ifdef HAVE_PTHREAD
 	pthread_mutex_unlock(&ifdh_context_mutex);
 #endif
@@ -971,7 +990,7 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		{
 			pcsc_tlv -> tag = FEATURE_VERIFY_PIN_DIRECT;
 			pcsc_tlv -> length = 0x04; /* always 0x04 */
-			pcsc_tlv -> value = IOCTL_FEATURE_VERIFY_PIN_DIRECT;
+			pcsc_tlv -> value = htonl(IOCTL_FEATURE_VERIFY_PIN_DIRECT);
 
 			pcsc_tlv++;
 			iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
@@ -982,7 +1001,7 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		{
 			pcsc_tlv -> tag = FEATURE_MODIFY_PIN_DIRECT;
 			pcsc_tlv -> length = 0x04; /* always 0x04 */
-			pcsc_tlv -> value = IOCTL_FEATURE_MODIFY_PIN_DIRECT;
+			pcsc_tlv -> value = htonl(IOCTL_FEATURE_MODIFY_PIN_DIRECT);
 
 			pcsc_tlv++;
 			iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
@@ -1097,17 +1116,25 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 
 		unsigned char res[10];
 		unsigned int length_res = sizeof(res);
+		RESPONSECODE ret;
 
 		/* if DEBUG_LEVEL_PERIODIC is not set we remove DEBUG_LEVEL_COMM */
 		oldLogLevel = LogLevel;
 		if (! (LogLevel & DEBUG_LEVEL_PERIODIC))
 			LogLevel &= ~DEBUG_LEVEL_COMM;
 
-		CmdEscape(reader_index, cmd, sizeof(cmd), res, &length_res);
+		ret = CmdEscape(reader_index, cmd, sizeof(cmd), res, &length_res);
 
 		/* set back the old LogLevel */
 		LogLevel = oldLogLevel;
 
+		if (ret != IFD_SUCCESS)
+		{
+			DEBUG_INFO("CmdEscape failed");
+			/* simulate a card absent */
+			res[0] = 0;
+		}
+		
 		if (0x01 == res[0])
 			return_value = IFD_ICC_PRESENT;
 		else
